// SkyBox Class
// Author: Michael Musson
// Date:  Nov 26, 2005

#include "skybox.h"

SkyBox::SkyBox(void)
{
  //NO-OP
}

SkyBox::~SkyBox(void)
{
  //NO-OP
}

SkyBox::SkyBox(char *inimgfilename)
{
  int i;
  
  scale = 1.0f;
  
  moving = 0.0f;
  
  skyDisplacement = 0.0f;
	moonDisplacement = 10.0f;

	char moonfile[] = "tga/moon.tga";

  // Create texture instances
  for(i=0; i<6; i++)
    {
    //SkyTexture[i] = new Texture();
    if ( this->SkyTexture[i].Init(&inimgfilename[i*32],0) == false )
      printf("Terrain::myTexture->Init(): Error loading TGA file: %s \n", inimgfilename[i]);
    }

	// Create moon texture instance
	if ( this->MoonTexture.Init(moonfile,0) == false )
		printf("Terrain::myTexture->Init(): Error loading: %s\n",moonfile);
 
  // Set up face coordinates
  faces[0][0] = 5; faces[0][1] = 1; faces[0][2] = 2; faces[0][3] = 6;
  faces[1][0] = 5; faces[1][1] = 4; faces[1][2] = 0; faces[1][3] = 1;
  faces[2][0] = 0; faces[2][1] = 4; faces[2][2] = 7; faces[2][3] = 3;
  faces[3][0] = 4; faces[3][1] = 5; faces[3][2] = 6; faces[3][3] = 7;
  faces[4][0] = 1; faces[4][1] = 0; faces[4][2] = 3; faces[4][3] = 2;
  faces[5][0] = 2; faces[5][1] = 3; faces[5][2] = 7; faces[5][3] = 6;

    /*
    {5, 1, 2, 6}, // back
    {5, 4, 0, 1}, // bottom
    {0, 4, 7, 3}, // front
    {4, 5, 6, 7}, // right 
    {1, 0, 3, 2}, // left  
    {2, 3, 7, 6}  // top
    */

  // Set up max/min values
  v[0][0] = v[1][0] = v[2][0] = v[3][0] = -1;  // min x
  v[4][0] = v[5][0] = v[6][0] = v[7][0] =  1;  // max x
  v[0][1] = v[1][1] = v[4][1] = v[5][1] = -1;  // min y
  v[2][1] = v[3][1] = v[6][1] = v[7][1] =  1;  // max y
  v[0][2] = v[3][2] = v[4][2] = v[7][2] = -1;  // min z
  v[1][2] = v[2][2] = v[5][2] = v[6][2] =  1;  // max z

	// Moon
	mv[0][0] = -1;
	mv[0][1] = -1;
	mv[1][0] = 1;
	mv[1][1] = -1;
	mv[2][0] = 1;
	mv[2][1] = 1;
	mv[3][0] = -1;
	mv[3][1] = 1;
	mv[0][2] = mv[1][2] = mv[2][2] = mv[3][2] = 0;

}


void SkyBox::DrawSkyBox(GLfloat scalex, GLfloat scaley, GLfloat scalez,
                        GLfloat centerx, GLfloat centery, GLfloat centerz)
{
  glPushMatrix();
  glTranslatef(centerx, centery, centerz);
  

  glRotatef( moving*skyDisplacement*100, 0,1,0 );

  glScalef(scalex,scaley,scalez);
  
  glColor4f( 1,1,1,1 );
  glMatrixMode( GL_TEXTURE );
  glPushMatrix();
  

  glTranslatef( 0,moving*skyDisplacement,0);

  glScalef( scale,scale,scale );
  //glRotatef( skyDisplacement, 1,0,0);
  for (int i = 5; i >= 0; i--) 
  {
    glBindTexture(GL_TEXTURE_2D, SkyTexture[i].GetID());
    glBegin(GL_QUADS);
    {
      
      glTexCoord2f(0,0);  glVertex3fv(&this->v[this->faces[i][0]][0]);
      glTexCoord2f(1,0);  glVertex3fv(&this->v[this->faces[i][1]][0]);
      glTexCoord2f(1,1);  glVertex3fv(&this->v[this->faces[i][2]][0]);
      glTexCoord2f(0,1);  glVertex3fv(&this->v[this->faces[i][3]][0]);
      
    }
    glEnd();
  }

	glPopMatrix();
	glMatrixMode( GL_MODELVIEW);
	glPopMatrix();


	/////////// MOOON //////////////
  glPushMatrix();

  glTranslatef( -moving*moonDisplacement*1.2-8.0 + centerx, centery + 4.0, centerz-20.0);
  glScalef(2.0,2.0,2.0);
 
 // glColor4f( 1,1,1,1 );
  glMatrixMode( GL_TEXTURE );	

	glPushMatrix();

	/* Eight symbolic constants are accepted: GL_ZERO, GL_ONE, GL_SRC_COLOR, GL_ONE_MINUS_SRC_COLOR, GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, GL_DST_ALPHA, and GL_ONE_MINUS_DST_ALPHA. */
	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );

	glEnable(GL_BLEND);

		glBindTexture(GL_TEXTURE_2D, MoonTexture.GetID());
		glBegin(GL_QUADS);
		
			glTexCoord2f(0,0); glVertex3fv(&mv[0][0]);
			glTexCoord2f(1,0); glVertex3fv(&mv[1][0]);
			glTexCoord2f(1,1); glVertex3fv(&mv[2][0]);
			glTexCoord2f(0,1); glVertex3fv(&mv[3][0]);
		 glEnd();
	glPopMatrix();
  
	glBlendFunc( GL_SRC_ALPHA, GL_DST_ALPHA );

  glMatrixMode( GL_MODELVIEW );
  glPopMatrix();
  
  skyDisplacement -= 0.0015f + 0.00025f*((float)rand()/(float)RAND_MAX);
	moonDisplacement -= 0.12;
}

void SkyBox::setScale( float theScale )
{
	scale = theScale;
}

void SkyBox:: setMovement( float theMoving )
{
	moving = theMoving;
}


